/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.sensor;

import java.awt.Dialog;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.List;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.SwingWorker;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.DefaultTableModel;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.model.Field;
import nl.cloudfarming.client.model.SensorRun;
import org.netbeans.api.progress.ProgressHandle;
import org.netbeans.api.progress.ProgressHandleFactory;
import org.openide.DialogDescriptor;
import org.openide.DialogDisplayer;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionReferences;
import org.openide.awt.ActionRegistration;
import org.openide.filesystems.FileChooserBuilder;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;

/**
 * Controller for exporting SensoRrun data (NDVI/WDVI Averages) in CSV-format
 * @author Gerben Feenstra
 */
@ActionID(id = "nl.cloudfarming.client.sensor.ExportSensorRunAveragesAction", category = "Tools/Sensor") //category = "Build", 
@ActionRegistration(iconBase = "nl/cloudfarming/client/sensor/sensor-export.png", displayName = "#action.exportsensorrunaverages.menu.displayname")
@ActionReferences({
    @ActionReference(path = "Menu/Tools/Sensor", position = 300)
})
public class ExportSensorRunController implements ActionListener, ListSelectionListener {

    private static final String HEADER_FIELD = NbBundle.getMessage(SensorModule.class, "action.exportsensorrunaverages.result.header.field");
    private static final String HEADER_PARTFIELD = NbBundle.getMessage(SensorModule.class, "action.exportsensorrunaverages.result.header.partfield");
    private static final String HEADER_SENSORRUN = NbBundle.getMessage(SensorModule.class, "action.exportsensorrunaverages.result.header.sensorrun");
    private static final String HEADER_NDVI = NbBundle.getMessage(SensorModule.class, "action.exportsensorrunaverages.result.header.ndvi");
    private static final String HEADER_WDVI = NbBundle.getMessage(SensorModule.class, "action.exportsensorrunaverages.result.header.wdvi");
    protected static final String[] HEADER = new String[]{HEADER_FIELD, HEADER_PARTFIELD, HEADER_SENSORRUN, HEADER_NDVI, HEADER_WDVI};
    private static final String JBUTTON_CANCEL = NbBundle.getMessage(SensorModule.class, "action.exportsensorrunaverages.panelcomponent.jbutton.cancel");
    private static final String JBUTTON_OK = NbBundle.getMessage(SensorModule.class, "action.exportsensorrunaverages.panelcomponent.jbutton.OK");
    private static final String DIALOG_TITLE = NbBundle.getMessage(SensorModule.class, "action.exportsensorrunaverages.dialog.title");
    private static final AppLogger LOG = AppLogFactory.getLogger(ExportSensorRunController.class);
    private static String FILECHOOSER_TITLE = NbBundle.getMessage(SensorModule.class, "action.exportsensorrunaverages.filechooser.title");
    private static String FILECHOOSER_OK = NbBundle.getMessage(SensorModule.class, "action.exportsensorrunaverages.filechooser.ok");
    private static String FILECHOOSER_FILE_DESC = NbBundle.getMessage(SensorModule.class, "action.exportsensorrunaverages.filechooser.filedescription");
    private FileChooserBuilder fcb;
    private ExportSensorRunPanel panel = null;
    private SensorService service;
    private Dialog dlg;
    private JButton jButtonOK;
    private JButton jButtonCancel;
    private static final String CSV_SEPARATOR = ",";
    private static final String DECIMAL_FORMAT = NbBundle.getMessage(SensorModule.class, "action.exportsensorrunaverages.dviparser.decimalformat");

    /**
     * For unit testing purposes, do not use!
     */
    protected void setService(SensorService service) {
        this.service = service;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == getPanel().getStartDatePicker() || e.getSource() == getPanel().getEndDatePicker()) {
            refreshFieldListModel();
        } else {
            service = Lookup.getDefault().lookup(SensorService.class);

            //Make a new panel. This is done so every time the screen is blank
            panel = new ExportSensorRunPanel();

            panel.getFieldList().addListSelectionListener(this);

            // Register listeners on Date-pickers
            panel.getStartDatePicker().addActionListener(this);
            panel.getEndDatePicker().addActionListener(this);

            JButton exportButton = getExportButton();
            JButton cancelButton = getCancelButton();

            DialogDescriptor dd = new DialogDescriptor(getPanel(), DIALOG_TITLE);
            dd.setOptions(new Object[]{exportButton, cancelButton});
            dlg = DialogDisplayer.getDefault().createDialog(dd);
            dlg.setVisible(true);
        }
    }

    /**
     * Listens to changes on the field list
     * @param e 
     */
    @Override
    public void valueChanged(ListSelectionEvent e) {
        refreshExampleTableModel();
    }

    public ExportSensorRunPanel getPanel() {
        if (panel == null) {
            panel = new ExportSensorRunPanel();
        }
        return panel;
    }

    /**
     * Creates the dialog cancel button or returns the already existing one.
     * @return Cancel JButton
     */
    private JButton getCancelButton() {
        if (jButtonCancel == null) {
            jButtonCancel = new JButton(JBUTTON_CANCEL);
        }
        return jButtonCancel;
    }

    /**
     * Creates the dialog Export button
     *
     * @return JButton ExportButton
     */
    private JButton getExportButton() {
        if (jButtonOK == null) {
            jButtonOK = new JButton();
            jButtonOK.setText(JBUTTON_OK);
            jButtonOK.addActionListener(new ActionListener() {

                @Override
                public void actionPerformed(ActionEvent ae) {
                    writeToFile();
                }
            });
        }
        return jButtonOK;
    }

    /**
     * Get the save location for the file.
     * pops up a save dialog
     * @return a file from the save dialog
     */
    public File getSaveLocation() {
        if (fcb == null) {

            fcb = new FileChooserBuilder("Export-directory");
            fcb.setTitle(FILECHOOSER_TITLE);
            fcb.setApproveText(FILECHOOSER_OK);
            fcb.addFileFilter(new FileFilter() {

                @Override
                public boolean accept(File pathname) {
                    if (pathname.isDirectory() && pathname.getName().startsWith(".")) {
                        return false;
                    }
                    return true;
                }

                @Override
                public String getDescription() {
                    return FILECHOOSER_FILE_DESC;
                }
            });
        }
        File workingDirectory = new File(SensorPreference.LAST_EXPORT_DIR.getValue());
        fcb.setDefaultWorkingDirectory(workingDirectory);
        return fcb.showSaveDialog();
    }

    /**
     * Refreshes the field list model.
     * This is a performance intensive method so an wait cursor is displayed.
     */
    protected void refreshFieldListModel() {
        DefaultListModel fieldListModel = (DefaultListModel) getPanel().getFieldList().getModel();

        List<Field> result = service.findAllFieldsWithSensorRunsByDate(getPanel().getStartDatePicker().getDate(), getPanel().getEndDatePicker().getDate());

        Field[] availableFields = result.toArray(new Field[]{});

        fieldListModel.removeAllElements();

        for (Field field : availableFields) {
            fieldListModel.addElement(field);
        }

    }

    public Field[] getSelectedFields() {
        Object[] selectedValues = getPanel().getFieldList().getSelectedValues();
        Field[] selectedFields = new Field[selectedValues.length];
        for (int i = 0; i < selectedValues.length; i++) {
            selectedFields[i] = (Field) selectedValues[i];
        }
        return selectedFields;
    }

    /**
     * Refreshes the field table model
     */
    private void refreshExampleTableModel() {
        Field[] fields = getSelectedFields();

        if (fields.length > 0) {
            String[][] result = getResultLinesForField(fields[0]);
            DefaultTableModel model = new DefaultTableModel(result, HEADER);
            getPanel().getExampleTable().setModel(model);
        }
    }

    /**
     * Write specified lines to specified file
     * Will log errors with error level.
     * 
     * @param lines LinupdateFieldModeles to write to file
     * @param file File to write to
     */
    public void writeToFile() {
        File file = getSaveLocation();
        File csvFile = null;
        if (file != null) {
            try {
                if (!file.getPath().endsWith(".csv")) {
                    csvFile = new File(file.getPath() + ".csv");
                } else {
                    csvFile = file;
                }
                ExportSensorRunFileWriter worker = new ExportSensorRunFileWriter(csvFile, getSelectedFields());
                worker.execute();
            } catch (IOException ex) {
                LOG.error("An exception occured while trying to write exported values to file", ex);
            }
        }
    }

    /**
     * Get sensor runs + their data for specified field as rows (arrays themselves) in an array
     * @param field Field for which to generate SensorRun report.
     * @return Multidimensional array with rows and their columns
     */
    protected String[][] getResultLinesForField(Field field) {
        List<SensorRun> sensorRuns = service.findSensorRunForField(field);

        if (!sensorRuns.isEmpty()) {

            int counter = sensorRuns.size();

            String[][] lines = new String[counter][HEADER.length];

            // Set DecimalFormat to #.###, so locale settings don't crash the application
            DecimalFormatSymbols dfs = new DecimalFormatSymbols();
            dfs.setDecimalSeparator('.');
            DecimalFormat decimalFormat = new DecimalFormat(DECIMAL_FORMAT, dfs);
            for (; counter > 0; counter--) {
                SensorRun sensorRun = sensorRuns.get(counter - 1);

                lines[counter - 1][0] = String.valueOf(field.getId()) + " " + field.getName();
                lines[counter - 1][1] = "";
                lines[counter - 1][2] = String.valueOf(sensorRun.getId());

                Double ndvi = sensorRun.getAverageNdvi();
                Double wdvi = sensorRun.getAverageWdvi();

                if (ndvi != null && ndvi != 0) {
                    ndvi = Double.parseDouble(decimalFormat.format(ndvi));
                    lines[counter - 1][3] = Double.toString(ndvi);
                }
                if (wdvi != null && wdvi != 0) {
                    wdvi = Double.parseDouble(decimalFormat.format(wdvi));
                    lines[counter - 1][4] = Double.toString(wdvi);
                }
            }
            return lines;
        }
        return null;
    }

    /**
     * Swing worker for writing the records to the file
     * @author Merijn Zengers
     */
    protected class ExportSensorRunFileWriter extends SwingWorker<File, String[]> {

        private final Field[] fields;
        private FileWriter fstream;
        private BufferedWriter out;
        private File csvFile;
        private ProgressHandle progressHandle;
        private final String PROCESS_TITLE = NbBundle.getMessage(SensorModule.class, "action.exportsensorrunaverages.export_sensor_run_file_writer.processing");
        private int progress = 0;
        
        public ExportSensorRunFileWriter(File csvFile, Field[] fields) throws IOException {
            this.csvFile = csvFile;
            this.fields = fields;
            fstream = new FileWriter(csvFile);
            out = new BufferedWriter(fstream);
            getExportButton().setEnabled(false);
            progressHandle = ProgressHandleFactory.createSystemHandle(PROCESS_TITLE);
        }

        @Override
        protected File doInBackground() throws Exception {
            progressHandle.start(fields.length);
            StringBuilder headerSb = new StringBuilder();
            for (String column : HEADER) {
                headerSb.append(column);
                headerSb.append(CSV_SEPARATOR);
            }

            //Remove last char
            headerSb.deleteCharAt(headerSb.length() - 1);
            headerSb.append("\n");
            out.write(headerSb.toString());

            for (Field field : fields) {
                for (String[] resultLine : getResultLinesForField(field)) {
                    publish((String[]) resultLine);
                }
            }
            return csvFile;
        }

        @Override
        protected void process(List<String[]> chunks) {
            for (String[] row : chunks) {
                try {
                    StringBuilder lineSb = new StringBuilder();
                    for (String cell : row) {
                        lineSb.append(cell);
                        lineSb.append(CSV_SEPARATOR);
                    }
                    //Remove last char
                    lineSb.deleteCharAt(lineSb.length() - 1);
                    lineSb.append("\n");
                    out.write(lineSb.toString());
                } catch (IOException ex) {
                    LOG.error("An exception occured while trying to write exported values to file", ex);
                }
                progressHandle.progress(progress++);
            }
        }

        @Override
        protected void done() {
            try {
                out.close();
                progressHandle.finish();
            } catch (IOException ex) {
                LOG.error("An exception occured while trying to close file", ex);
            }
        }
    }
}
