/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.sensor;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import javax.swing.filechooser.FileFilter;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.util.FilePickerUtil;
import org.apache.commons.io.FileUtils;
import org.netbeans.api.progress.ProgressHandle;
import org.netbeans.api.progress.ProgressHandleFactory;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionRegistration;
import org.openide.filesystems.FileChooserBuilder;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.LookupEvent;
import org.openide.util.LookupListener;
import org.openide.util.NbBundle;

/**
 *
 * @author Timon Veenstra
 */
@ActionID(id = "nl.cloudfarming.client.sensor.SensorFilePicker", category = "Tools/Sensor")
@ActionRegistration(iconInMenu = true, displayName = "#sensor.filepicker.menu.displayname", iconBase="nl/cloudfarming/client/sensor/loadsensor.png")
@ActionReference(path = "Menu/Tools/Sensor", position = 50)
public class SensorFilePicker implements ActionListener {

    private static final AppLogger LOG = AppLogFactory.getLogger(SensorFilePicker.class);
    private static final String FILEPICKER_TITLE = NbBundle.getMessage(SensorModule.class, "sensor.filepicker.title");
    private static final String FILEPICKER_APPROVE = NbBundle.getMessage(SensorModule.class, "sensor.filepicker.approve");
    private static final String FILEPICKER_PROCESS_TITLE = NbBundle.getMessage(SensorModule.class, "sensor.process.process.title");
    private static final String FILEPICKER_ALL_SUPPORTED_FILETYPES_TEXT = NbBundle.getMessage(SensorModule.class, "sensor.filepicker.all_supported_filetypes.text");
      
    private final Lookup.Result<SensorProcessorProvider> sensorProcessorProviderResult = Lookup.getDefault().lookupResult(SensorProcessorProvider.class);
    private final ProxyFileFilter filterAllSensors = new ProxyFileFilter();

    private final List<File> processQueue = Collections.synchronizedList(new ArrayList<File>());
    private final List<File> copyQueue = Collections.synchronizedList(new ArrayList<File>());

    public SensorFilePicker() {
        sensorProcessorProviderResult.addLookupListener(new LookupListener() {
            @Override
            public void resultChanged(LookupEvent ev) {
                synchronized (filterAllSensors) {
                    filterAllSensors.reset();
                    for (SensorProcessorProvider processorProvider : ((Lookup.Result<SensorProcessorProvider>) ev.getSource()).allInstances()) {
                        filterAllSensors.addFilter(FilePickerUtil.createSwingFileFilter(processorProvider.getSupportedExtentions(), processorProvider.getFormatDescription()));
                    }
                }
            }
        });
    }

    /**
     * 
     * @param e 
     */
    @Override
    public void actionPerformed(ActionEvent e) {
        // Retreive last workingdirectory
        File workingDirectory = new File(SensorPreference.LAST_DIR.getValue());

        // Build the filechooser
        FileChooserBuilder fcb = new FileChooserBuilder("Sensor-directory");
        fcb.setTitle(FILEPICKER_TITLE);
        fcb.setDefaultWorkingDirectory(workingDirectory);
        fcb.setApproveText(FILEPICKER_APPROVE);

        // Here we loop through all ProcessorProviders to retreive all possible extensions
        // also save them to supportedExtensions to build an "All sensor filetypes" filefilter from
        final ArrayList<String> supportedExtensions = new ArrayList<String>();
        for (SensorProcessorProvider processorProvider : Lookup.getDefault().lookupAll(SensorProcessorProvider.class)) {
            fcb.addFileFilter(FilePickerUtil.createSwingFileFilter(processorProvider.getSupportedExtentions(), processorProvider.getFormatDescription()));
            supportedExtensions.addAll(processorProvider.getSupportedExtentions());
        }
        
        // Create the "All sensor filetypes" filefilter
        fcb.addFileFilter(FilePickerUtil.createSwingFileFilter(supportedExtensions, FilePickerUtil.getFileFilterDescription(FILEPICKER_ALL_SUPPORTED_FILETYPES_TEXT, supportedExtensions)));

        // Open dialog, retreive selected sensorfile
        File selectedFile = fcb.showOpenDialog();

        if (selectedFile != null) {
            if (selectedFile.isDirectory()) {
                synchronized (filterAllSensors) {
                    copyQueue.addAll(Arrays.asList(selectedFile.listFiles(filterAllSensors)));
                }
            } else if (selectedFile.isFile()) {
                copyQueue.add(selectedFile);
            }
            // start copy process
            try {
                copy(copyQueue);
            } catch (IOException ex) {
                Exceptions.printStackTrace(ex);
            }
            // start processing of files
            Thread processThread = new Thread(new ProcessJob());
            processThread.start();
        }

    }

    /**
     * Copy queue to archive dir.
     * @param copyQueue List of files to copy to the archive dir.
     * @throws IOException 
     */
    protected void copy(List<File> copyQueue) throws IOException {

        synchronized (processQueue) {
            File archiveDir = new File(SensorPreference.ARCHIVE.getValue());
            if (!archiveDir.exists()) {
                archiveDir.mkdirs();
            }
            for (File file : copyQueue) {
                StringBuilder sb = new StringBuilder(archiveDir.getAbsolutePath());
                sb.append("/");
                sb.append(file.getName());
                File archivedSensorFile = new File(sb.toString());
                if (!file.getAbsolutePath().equals(archivedSensorFile.getAbsolutePath())) {
                    FileUtils.copyFile(file, archivedSensorFile, true);
                }
                processQueue.add(archivedSensorFile);
            }
            copyQueue.clear();
        }
    }

    private class ProxyFileFilter implements java.io.FileFilter {

        private final List<FileFilter> filters = new ArrayList<FileFilter>();

        @Override
        public boolean accept(File pathname) {
            for (FileFilter filter : filters) {
                if (filter.accept(pathname)) {
                    return true;
                }
            }
            return false;
        }

        public void reset() {
            filters.clear();
        }

        public void addFilter(FileFilter filter) {
            filters.add(filter);
        }
    }

    class ProcessJob implements Runnable {

        @Override
        public void run() {
            synchronized (processQueue) {
                // system handle will give this handle less priority diplaying file handles first
                ProgressHandle progressHandle = ProgressHandleFactory.createSystemHandle(FILEPICKER_PROCESS_TITLE);
                int progress = 0;
                progressHandle.start(processQueue.size());
                for (File file : processQueue) {

                    // Try finding a processor for the specified file
                    SensorProcessor processor = null;
                    SensorProcessorProvider foundProcessorProvider = null;
                    
                    findprocessor:
                    for (SensorProcessorProvider processorProvider : sensorProcessorProviderResult.allInstances()) {
                        if (processorProvider.isSupported(file) && processorProvider.isValid(file)) {
                            foundProcessorProvider = processorProvider;
                            processor = processorProvider.getProcessor(file);
                            break findprocessor;
                        }
                    }
                        
                    // Start process if a valid processor is found
                    if (foundProcessorProvider == null) {
                        LOG.warning(NbBundle.getMessage(SensorModule.class, "sensor.filepicker.file_could_not_be_processed", file.getName()));
                    }
                    else{
                        processor.process();
                    }
                    progressHandle.progress(++progress);

                }
                
                // Finish
                progressHandle.finish();
                processQueue.clear();
            }
        }
    }
}
