/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a copy
 * of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.sensor;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import nl.cloudfarming.client.geoviewer.Palette;
import nl.cloudfarming.client.geoviewer.ValueRange;
import nl.cloudfarming.client.model.SensorDataFile;
import nl.cloudfarming.client.model.SensorRun;
import org.netbeans.api.visual.model.ObjectState;
import org.openide.util.lookup.ServiceProvider;

/**
 * SensorPalette. The sensor palette contains different colors for different
 * sensor reading values. The sensor palette is interchangeable. It contains a
 * static list with palettes for different value ranges.
 *
 *
 * @author Timon Veenstra
 */
@ServiceProvider(service = Palette.class)
public class SensorPalette extends Palette<Number> implements ValueRange<Double> {

    private double rangeFrom;
    private double rangeTo;
    private int nrOfColors = 20;
    public static final int SENSOR_DATA_MULTIPLIER = 1000;
    private static final List<Palette<Number>> choiceList = new ArrayList<Palette<Number>>();
    public static final SensorPalette PALLET01_04 = new SensorPalette(0.1, 0.4);
    public static final SensorPalette PALLET01_06 = new SensorPalette(0.1, 0.6);
    public static final SensorPalette PALLET01_08 = new SensorPalette(0.1, 0.8);
    public static final SensorPalette PALLET02_04 = new SensorPalette(0.2, 0.4);
    public static final SensorPalette PALLET02_06 = new SensorPalette(0.2, 0.6);
    public static final SensorPalette PALLET02_08 = new SensorPalette(0.2, 0.8);
    public static final SensorPalette PALLET00_10 = new SensorPalette(0.1, 1.0);

    static {
        choiceList.add(PALLET01_04);
        choiceList.add(PALLET01_06);
        choiceList.add(PALLET01_08);
        choiceList.add(PALLET02_04);
        choiceList.add(PALLET02_06);
        choiceList.add(PALLET02_08);
        choiceList.add(PALLET00_10);
    }

    public SensorPalette() {
        super(Color.BLACK);
    }
    
    

    private SensorPalette(double rangeFrom, double rangeTo) {
        super(Color.BLACK);
        this.rangeFrom = rangeFrom;
        this.rangeTo = rangeTo;

    }

    public double getRangeFrom() {
        return rangeFrom;
    }

    public double getRangeTo() {
        return rangeTo;
    }

    @Override
    public String toString() {
        return "SensorDataPallet " + rangeFrom + " -> " + rangeTo;
    }

    @Override
    public Double getMinValue() {
        return rangeFrom * SENSOR_DATA_MULTIPLIER;
    }

    @Override
    public Double getMaxValue() {
        return rangeTo * SENSOR_DATA_MULTIPLIER;
    }

    @Override
    public int getSteps() {
        return nrOfColors;
    }

    @Override
    public boolean supports(Class clazz) {
        return (SensorDataFile.class.isAssignableFrom(clazz) || SensorRun.class.isAssignableFrom(clazz));
    }

    @Override
    public Color getColorForValue(Number value) {
        double dvalue = value.doubleValue();
        double step = (getRangeTo() - getRangeFrom()) / ((getSteps() - 1));

        if (dvalue < 0d) {
            return Color.BLACK;
        } else if (dvalue <= getRangeFrom() * SENSOR_DATA_MULTIPLIER) {
            return new Color(0x8F050C);
        } else if (dvalue <= (getRangeFrom() + (1 * step)) * SENSOR_DATA_MULTIPLIER) {
            return new Color(0xB60A06);
        } else if (dvalue <= (getRangeFrom() + (2 * step)) * SENSOR_DATA_MULTIPLIER) {
            return new Color(0xDB0F01);
        } else if (dvalue <= (getRangeFrom() + (3 * step)) * SENSOR_DATA_MULTIPLIER) {
            return new Color(0xE32600);
        } else if (dvalue <= (getRangeFrom() + (4 * step)) * SENSOR_DATA_MULTIPLIER) {
            return new Color(0xE24000);
        } else if (dvalue <= (getRangeFrom() + (5 * step)) * SENSOR_DATA_MULTIPLIER) {
            return new Color(0xE65E00);
        } else if (dvalue <= (getRangeFrom() + (6 * step)) * SENSOR_DATA_MULTIPLIER) {
            return new Color(0xF17F00);
        } else if (dvalue <= (getRangeFrom() + (7 * step)) * SENSOR_DATA_MULTIPLIER) {
            return new Color(0xFC9F00);
        } else if (dvalue <= (getRangeFrom() + (8 * step)) * SENSOR_DATA_MULTIPLIER) {
            return new Color(0xFDB703);
        } else if (dvalue <= (getRangeFrom() + (9 * step)) * SENSOR_DATA_MULTIPLIER) {
            return new Color(0xFAD007);
        } else if (dvalue <= (getRangeFrom() + (10 * step)) * SENSOR_DATA_MULTIPLIER) {
            return new Color(0xF7E80B);
        } else if (dvalue <= (getRangeFrom() + (11 * step)) * SENSOR_DATA_MULTIPLIER) {
            return new Color(0xDCF40A);
        } else if (dvalue <= (getRangeFrom() + (12 * step)) * SENSOR_DATA_MULTIPLIER) {
            return new Color(0xB7F805);
        } else if (dvalue <= (getRangeFrom() + (13 * step)) * SENSOR_DATA_MULTIPLIER) {
            return new Color(0x92FC00);
        } else if (dvalue <= (getRangeFrom() + (14 * step)) * SENSOR_DATA_MULTIPLIER) {
            return new Color(0x69E003);
        } else if (dvalue <= (getRangeFrom() + (15 * step)) * SENSOR_DATA_MULTIPLIER) {
            return new Color(0x41C405);
        } else if (dvalue <= (getRangeFrom() + (16 * step)) * SENSOR_DATA_MULTIPLIER) {
            return new Color(0x16A508);
        } else if (dvalue <= (getRangeFrom() + (17 * step)) * SENSOR_DATA_MULTIPLIER) {
            return new Color(0x048608);
        } else if (dvalue <= (getRangeFrom() + (18 * step)) * SENSOR_DATA_MULTIPLIER) {
            return new Color(0x046707);
        } else if (dvalue <= getRangeTo()) {
            return new Color(0x044606);
        }
        return Color.GRAY;
    }

    @Override
    public Color getColorForState(ObjectState state) {
        return Color.BLACK;
    }

    @Override
    public boolean isInterchangeable() {
        return true;
    }

    @Override
    public List<Palette<Number>> getInterchangeablePalettes() {
        return Collections.unmodifiableList(choiceList);
    }
}
